package de.ozgcloud.nachrichten.postfach.osiv2;

import static de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockserver.matchers.Times.*;
import static org.mockserver.model.Header.*;
import static org.mockserver.model.HttpRequest.*;
import static org.mockserver.model.HttpResponse.*;
import static org.mockserver.model.Parameter.*;
import static org.mockserver.model.ParameterBody.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.RegisterExtension;
import org.mockserver.client.MockServerClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.DynamicPropertyRegistry;
import org.springframework.test.context.DynamicPropertySource;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.extension.Jwt;
import de.ozgcloud.nachrichten.postfach.osiv2.extension.OsiMockServerExtension;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory;
import lombok.SneakyThrows;

@SpringBootTest(classes = TestApplication.class, webEnvironment = SpringBootTest.WebEnvironment.NONE)
public class OsiPostfachRemoteServiceITCase {

	@RegisterExtension
	static final OsiMockServerExtension OSI_MOCK_SERVER_EXTENSION = new OsiMockServerExtension();

	private static final String MESSAGE_ID = "message-id";
	private final PostfachNachricht postfachNachricht = PostfachNachricht.builder()
			.messageId(MESSAGE_ID)
			.build();

	@Autowired
	private OsiPostfachRemoteService osiPostfachRemoteService;

	@DynamicPropertySource
	static void dynamicProperties(DynamicPropertyRegistry registry) {
		registry.add("spring.security.oauth2.client.provider.osi2.token-uri", OSI_MOCK_SERVER_EXTENSION::getAccessTokenUrl);
		registry.add("spring.security.oauth2.client.registration.osi2.scope", () -> CLIENT_SCOPES);
		registry.add("spring.security.oauth2.client.registration.osi2.client-id", () -> CLIENT_ID);
		registry.add("ozgcloud.osiv2-postfach.api.url", OSI_MOCK_SERVER_EXTENSION::getPostfachFacadeUrl);
		registry.add("ozgcloud.osiv2-postfach.api.resource", () -> RESOURCE_URN);
	}

	private MockServerClient mockClient;

	@BeforeEach
	@SneakyThrows
	public void setup() {
		mockClient = OSI_MOCK_SERVER_EXTENSION.getMockClient();

		mockClient
				.when(
						request()
								.withMethod("POST")
								.withPath("/access-token")
								.withHeaders(
										header("Content-Type", "application/x-www-form-urlencoded;charset=UTF-8")
								)
								.withBody(
										params(
												param("grant_type", "client_credentials"),
												param("client_id", CLIENT_ID),
												param("client_secret", "changeme"),
												param("scope", String.join(" ", CLIENT_SCOPES)),
												param("resource", RESOURCE_URN)
										)
								),
						exactly(1)
				)
				.respond(
						response()
								.withStatusCode(200)
								.withHeader("Content-Type", "application/json")
								.withBody(
										JwtFactory.createTokenResponse(
												JwtFactory.createAccessTokenExampleWithExpireIn(900)
										)
								)
				);
	}

	@DisplayName("send message")
	@Nested
	class TestSendMessage {

		@DisplayName("should send dummy request with jwt")
		@Test
		void shouldSendDummyRequestWithJwt() {
			mockClient
					.when(
							request()
									.withMethod("GET")
									.withPath("/dummy"),
							exactly(1)
					)
					.respond(
							response()
									.withStatusCode(200)
					);

			osiPostfachRemoteService.sendMessage(postfachNachricht);

			var requests = mockClient.retrieveRecordedRequests(
					request()
							.withMethod("GET")
							.withPath("/dummy")
			);
			assertThat(requests).hasSize(1);
			var jwt = Jwt.parseAuthHeaderValue(
					requests[0].getHeader("Authorization").getFirst()
			);
			assertThat(jwt.body().read("$.client_id", String.class)).isEqualTo(CLIENT_ID);
			assertThat(jwt.body().read("$.aud", String.class)).isEqualTo(RESOURCE_URN);
		}

	}
}
